;;; ports - package additions for infotropique OS
;;; Copyright (C) 2017, 2018 ng0 <gillmann@infotropique.org>
;;;
;;; This program is free software: you can redistribute it and/or modify
;;; it under the terms of the GNU Affero General Public License as
;;; published by the Free Software Foundation, either version 3 of the
;;; License, or (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU Affero General Public License for more details.
;;;
;;; You should have received a copy of the GNU Affero General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

;; Parts of this file originated from the (gnu packages gnuzilla) module of
;; GNU Guix which is GPL3 licensed.

(define-module (mail thunderbird thunderbird)
  #:use-module (guix build-system gnu)
  #:use-module (guix download)
  #:use-module ((guix licenses) #:prefix license:)
  #:use-module (guix packages)
  #:use-module (gnu packages)
  #:use-module (gnu packages gnuzilla)
  #:use-module (gnu packages rust)
  #:use-module ((srfi srfi-1) #:hide (zip))
  #:use-module (ice-9 match)
  #:use-module (guix utils)
  #:use-module (guix gexp) ;local-file
  #:use-module (gnu packages autotools)
  #:use-module (gnu packages base)
  #:use-module (gnu packages databases)
  #:use-module (gnu packages glib)
  #:use-module (gnu packages gtk)
  #:use-module (gnu packages gnome)
  #:use-module (gnu packages libcanberra)
  #:use-module (gnu packages cups)
  #:use-module (gnu packages kerberos)
  #:use-module (gnu packages linux)
  #:use-module (gnu packages perl)
  #:use-module (gnu packages pkg-config)
  #:use-module (gnu packages compression)
  #:use-module (gnu packages fontutils)
  #:use-module (gnu packages libevent)
  #:use-module (gnu packages libreoffice)  ;for hunspell
  #:use-module (gnu packages image)
  #:use-module (gnu packages libffi)
  #:use-module (gnu packages pulseaudio)
  #:use-module (gnu packages python)
  #:use-module (gnu packages xorg)
  #:use-module (gnu packages gl)
  #:use-module (gnu packages assembly)
  #:use-module (gnu packages icu4c)
  #:use-module (gnu packages video)
  #:use-module (gnu packages xdisorg)
  #:use-module (gnu packages readline)
  #:use-module (gnu packages rust))

;; patches location is relative to the top directory, if you see what i mean
;; alternatively (patches (list (local-file "patches/foo.patch")))

(define-public thunderbird
  (package 
    (name "thunderbird")
    (version "52.6.0")
    (source
     (origin
       (method url-fetch)
       (uri (string-append "https://archive.mozilla.org/pub/"
                           "thunderbird/releases/"
                           version "/source/thunderbird-" version
                           ".source.tar.xz"))
       ;; (patches (list
       ;;           ;;(local-file "patches/thunderbird-52.2.1-dont-hardcode-libc-soname.patch")
       ;;           ;;(local-file "patches/thunderbird-52.2.1-fix-hardened-pie-detection.patch")
       ;;           ;;"patches/thunderbird-52.2.1-system-graphite2.patch"
       ;;           ;;"patches/thunderbird-52.2.1-system-harfbuzz.patch"
       ;;           ;;(local-file "patches/thunderbird-52.2.1-use-c99-math-isfinite.patch")
       ;;           ;;(local-file "patches/icedove/no-crmf.patch")
       ;;           (local-file "patches/icedove/rust-fix-rusty-cheddar-crate-setup.patch")
       ;;           (local-file "patches/icedove/rust-target-i586-to-i686.patch")
       ;;           ;;"patches/icedove/libre-searchengines.patch"
       ;;           ;;"patches/icedove/no-neon.patch"))
       ;;           ))

       (sha256
        (base32
         "1a25a02mcxm358nmbwcfdm7jj639xlx1zp7a81d7wp3nlfsmy88n"))))
    (build-system gnu-build-system)
    (arguments
     `(#:tests? #f          ; no check target
       ;;#:out-of-source? #t  ; must be built outside of the source directory

       ;; XXX: There are RUNPATH issues such as
       ;; $prefix/lib/icecat-31.6.0/plugin-container NEEDing libmozalloc.so,
       ;; which is not in its RUNPATH, but they appear to be harmless in
       ;; practice somehow.  See <http://hydra.gnu.org/build/378133>.
       #:validate-runpath? #f

       ;;#:parallel-build? #f

       #:configure-flags '(;(string-append "CPPFLAGS=-I"
                           ;;               "db/mork/public")
                           "--enable-application=mail"
                           "--disable-alsa"
                           "--disable-pulseaudio"

                           "--with-system-jpeg"
                           "--with-system-zlib"
                           "--with-system-bz2"
                           "--with-system-nspr"
                           "--with-system-nss"
                           "--with-system-libevent"
                           ;;"--with-system-png" # needs APNG support
                           "--with-system-icu"
                           "--enable-system-ffi"
                           "--enable-system-hunspell"
                           "--enable-system-pixman"
                           "--enable-system-sqlite"
                           ;;"--enable-system-cairo"
                           "--enable-startup-notification"
                           ;;"--enable-content-sandbox" ; available since 26.0, but not much info available
                           "--disable-crashreporter"
                           "--disable-tests"
                           "--disable-necko-wifi"
                           "--disable-updater"
                           "--enable-jemalloc"
                           "--disable-gconf"
                           "--enable-default-toolkit=cairo-gtk2"
                           "--enable-calendar"
                           "--disable-debug"
                           "--enable-release"
                           "--disable-debug-symbols"
                           "--enable-optimize"
                           "--enable-strip"
                           "--disable-rust"
                           "--disable-official-branding")
                           ;;"--with-user-appdir=.thunderbird")

       #:modules ((ice-9 ftw)
                  (ice-9 rdelim)
                  (ice-9 match)
                  ,@%gnu-build-system-modules)
       #:phases
       (modify-phases %standard-phases
         (add-after 'unpack 'ensure-no-mtimes-pre-1980
           (lambda _
             ;; Without this, the 'source/test/addons/packed.xpi' and
             ;; 'source/test/addons/simple-prefs.xpi' targets fail while trying
             ;; to create zip archives.
             (let ((early-1980 315619200)) ; 1980-01-02 UTC
               (ftw "." (lambda (file stat flag)
                          (unless (<= early-1980 (stat:mtime stat))
                            (utime file early-1980 early-1980))
                          #t))
               #t)))
         ;; (replace 'configure
         ;;  ;; configure does not work followed by both "SHELL=..." and
         ;;  ;; "CONFIG_SHELL=..."; set environment variables instead
         ;;  (lambda* (#:key outputs configure-flags #:allow-other-keys)
         ;;    (let* ((out (assoc-ref outputs "out"))
         ;;           (bash (which "bash"))
         ;;           (abs-srcdir (getcwd))
         ;;           (srcdir (string-append "../" (basename abs-srcdir)))
         ;;           (flags `(,(string-append "--prefix=" out)
         ;;                    ,(string-append "--with-l10n-base="
         ;;                                    abs-srcdir "/l10n")
         ;;                    ,@configure-flags))
         ;;           (mozconfig (string-append (getcwd) "/.mozconfig")))
         ;;      (setenv "SHELL" bash)
         ;;      (setenv "CONFIG_SHELL" bash)
         ;;      (setenv "AUTOCONF" (which "autoconf")) ; must be autoconf-2.13
         ;;      (mkdir "../build")
         ;;      (chdir "../build")
         ;;      (format #t "build directory: ~s~%" (getcwd))
         ;;      (format #t "configure flags: ~s~%" flags)
         ;;      (zero? (apply system* bash
         ;;                    (string-append srcdir "/configure")
         ;;                    flags)))))
         (add-before 'configure 'dont-exit-with-error-on-libs
           (lambda _
             (substitute* "mail/installer/Makefile.in"
               (("MOZ_PKG_FATAL_WARNINGS = 1")
                "MOZ_PKG_FATAL_WARNINGS = 0"))
             #t))
         ;; (add-before 'configure 'strip-faceboogle
         ;;   (lambda _
         ;;     (substitute* "chat/moz.build"
         ;;       (("'protocols/facebook',") "")
         ;;       (("'protocols/gtalk',") "")
         ;;       (("'protocols/odnoklassniki',") "")
         ;;       (("'protocols/twitter',") "")
         ;;       (("'protocols/yahoo',") ""))
         ;;     (substitute* "mail/installer/package-manifest.in"
         ;;       (("facebook.js") "")
         ;;       (("facebook.manifest") "")
         ;;       (("gtalk.js") "")
         ;;       (("gtalk.manifest") "")
         ;;       (("twitter.js") "")
         ;;       (("twitter.manifest") "")
         ;;       (("yahoo.js") "")
         ;;       (("yahoo.manifest") ""))
         ;;     (for-each delete-file-recursively
         ;;               '("chat/protocols/facebook"
         ;;                 "chat/protocols/gtalk"
         ;;                 "chat/protocols/twitter"
         ;;                 "chat/protocols/yahoo"))
         ;;     #t))
         ;; (add-before 'configure 'parabola-remove-url-classifier-to-build-and-disable-phishing-protection
         ;;   (lambda _
         ;;     (substitute* "mail/installer/package-manifest.in"
         ;;       (("Phishing Protection") "")
         ;;       (("UrlClassifier") "")
         ;;       (("URLClassifier") "")
         ;;       (("url-classifier") ""))
         ;;     #t))
         ;; (add-before 'configure 'arrrrrgh
         ;;   (lambda* (#:key outputs #:allow-other-keys)
         ;;     (let* ((out (assoc-ref outputs "out"))
         ;;            (inc (string-append out "/include"))
         ;;            (mdb.h (string-append inc "/mdb.h"))
         ;;            (nsMorkCID.h (string-append inc "/nsMorkCID.h")))
         ;;       (mkdir-p inc)
         ;;       (install-file "db/mork/public/mdb.h" inc)
         ;;       (install-file "db/mork/build/nsMorkCID.h" inc)
         ;;       (with-directory-excursion "db/mork/src"
         ;;         (substitute* (find-files "." "^.cpp$")
         ;;           (("mdb.h")
         ;;            mdb.h)))
         ;;       (substitute* "db/mork/build/nsMorkFactory.cpp"
         ;;         (("mdb.h")
         ;;          mdb.h))
         ;;       (substitute* (list "mailnews/import/oexpress/nsOEAddressIterator.h"
         ;;                          "mailnews/import/outlook/src/nsOutlookMail.cpp"
         ;;                          "mailnews/import/text/src/nsTextAddress.cpp"
         ;;                          "mailnews/addrbook/src/nsAbMDBDirProperty.cpp"
         ;;                          "mailnews/addrbook/src/nsAbLDIFService.cpp"
         ;;                          "mailnews/addrbook/src/nsAbMDBDirectory.cpp"
         ;;                          "mailnews/addrbook/src/nsAddrDatabase.h"
         ;;                          "mailnews/base/src/nsMsgFolderCache.h"
         ;;                          "mailnews/base/src/nsMsgFolderCacheElement.h"
         ;;                          "mailnews/db/msgdb/public/nsMsgThread.h"
         ;;                          "mailnews/db/msgdb/public/nsMsgHdr.h"
         ;;                          "mailnews/db/msgdb/public/nsDBFolderInfo.h"
         ;;                          "mailnews/db/msgdb/src/nsMsgOfflineImapOperation.h")
         ;;         (("mdb.h")
         ;;          mdb.h))
         ;;       (substitute* (list "mailnews/addrbook/src/nsAddrDatabase.cpp"
         ;;                          "mailnews/base/src/nsMsgFolderCache.cpp"
         ;;                          "mailnews/db/msgdb/src/nsMsgDatabase.cpp"
         ;;                          "db/mork/build/nsMorkFactory.cpp")
         ;;         (("nsMorkCID.h")
         ;;          nsMorkCID.h))

         ;;       #t)))
         (add-after 'unpack 'use-skia-by-default
           (lambda _
             ;; Use the bundled Skia library by default, since IceCat appears
             ;; to be far more stable when using it than when using our system
             ;; Cairo.
             (let ((out (open "mail/app/profile/prefs.js"
                              (logior O_WRONLY O_APPEND))))
               (format out "~%// Use Skia by default~%")
               (format out "pref(~s, ~s);~%" "gfx.canvas.azure.backends" "skia")
               (format out "pref(~s, ~s);~%" "gfx.content.azure.backends" "skia")
               (close-port out))
             #t))
         (replace 'configure
           (lambda* (#:key outputs configure-flags #:allow-other-keys)
             (let* ((out (assoc-ref outputs "out"))
                    (bash (which "bash"))
                    (abs-srcdir (getcwd))
                    (srcdir (string-append "../" (basename abs-srcdir)))
                    (flags `(,(string-append "--prefix=" out)
                             ,@configure-flags))
                    (mozconfig (string-append (getcwd) "/.mozconfig")))
               (setenv "SHELL" bash)
               (setenv "AUTOCONF"
                       (string-append (assoc-ref %build-inputs
                                                 "autoconf")
                                      "/bin/autoconf"))
               (setenv "CONFIG_SHELL" bash)
               (setenv "QA_CONFIGURE_OPTIONS" ".*")
               (setenv "MOZBUILD_STATE_PATH"
                       (string-append (getcwd) "/mach_state"))
               (setenv "MOZCONFIG"
                       (string-append (getcwd) "/.mozconfig"))
               (setenv "MOZ_NOSPAM" "1")
               (setenv "PYTHON"
                       (string-append (assoc-ref %build-inputs
                                                 "python-2.7")
                                      "/bin/python2"))
               ;; (setenv "builddir" (string-append (getcwd) "/build"))
               ;; (mkdir-p (getenv "MOZBUILD_STATE_PATH"))
               (mkdir-p (string-append (getcwd) "/builddir"))
               (with-output-to-file mozconfig
                 (lambda ()
                   (display
                    (string-append
                     "ac_add_options --enable-application=mail\n"
                     "ac_add_options --prefix=" out "\n"
                     "ac_add_options --disable-pulseaudio\n"
                     "ac_add_options --with-system-jpeg\n"
                     "ac_add_options --with-system-zlib\n"
                     "ac_add_options --with-system-bz2\n"
                     "ac_add_options --with-system-nspr\n"
                     "ac_add_options --with-system-nss\n"
                     "ac_add_options --with-system-libevent\n"
                     "ac_add_options --with-system-icu\n"
                     "ac_add_options --enable-system-ffi\n"
                     "ac_add_options --enable-system-hunspell\n"
                     "ac_add_options --enable-system-pixman\n"
                     "ac_add_options --enable-system-sqlite\n"
                     "ac_add_options --enable-startup-notification\n"
                     "ac_add_options --enable-content-sandbox\n"
                     "ac_add_options --disable-crashreporter\n"
                     "ac_add_options --disable-tests\n"
                     "ac_add_options --disable-necko-wifi\n"
                     "ac_add_options --disable-updater\n"
                     ;;"ac_add_options --enable-jemaloc\n"
                     "ac_add_options --disable-gconf\n"
                     "ac_add_options --enable-default-toolkit=\"cairo-gtk2\"\n"
                     "ac_add_options --enable-calendar\n"
                     "ac_add_options --disable-debug\n"
                     "ac_add_options --disable-debug-symbols\n"
                     "ac_add_options --enable-optimize\n"
                     "ac_add_options --enable-release\n"
                     "ac_add_options --enable-strip\n"
                     "ac_add_options --enable-gold\n"
                     "ac_add_options --disable-rust\n"
                     "ac_add_options --with-distribution-id=org.gnu\n"
                     "ac_add_options --disable-elf-hack\n"
                     ;;"ac_add_options --disable-neon\n"
                     "ac_add_options --disable-ion\n"
                     "ac_add_options --disable-webrtc\n"
                     "ac_add_options --disable-official-branding\n"
                     "ac_add_options --with-user-appdir=\\.thunderbird\n"))))
               (display (getcwd))
               (newline)
               (display "mach configure")
               (zero? (system* "./mozilla/mach" "configure"))
               ;;(chdir "builddir")
               (display "mach")
               (zero? (system* "./mozilla/mach" "build"))))))))
         (inputs
          `(("alsa-lib" ,alsa-lib)
            ("bzip2" ,bzip2)
            ("cairo" ,cairo)
            ("cups" ,cups)
            ("dbus-glib" ,dbus-glib)
            ("gdk-pixbuf" ,gdk-pixbuf)
            ("glib" ,glib)
            ("gtk+" ,gtk+)
            ("gtk+-2" ,gtk+-2)
            ("pango" ,pango)
            ("freetype" ,freetype)
            ("hunspell" ,hunspell)
            ("libcanberra" ,libcanberra)
            ("libgnome" ,libgnome)
            ("libjpeg-turbo" ,libjpeg-turbo)
            ("libpng-apng" ,libpng-apng)
            ("libxft" ,libxft)
            ("libevent" ,libevent-2.0)
            ("libxinerama" ,libxinerama)
            ("libxscrnsaver" ,libxscrnsaver)
            ("libxcomposite" ,libxcomposite)
            ("libxt" ,libxt)
            ("libffi" ,libffi)
            ("ffmpeg" ,ffmpeg)
            ("libvpx" ,libvpx)
            ("icu4c" ,icu4c)
            ("pixman" ,pixman)
            ("pulseaudio" ,pulseaudio)
            ("mesa" ,mesa)
            ("mit-krb5" ,mit-krb5)
            ("nspr" ,nspr)
            ("nss" ,nss)
            ("sqlite" ,sqlite)
            ("startup-notification" ,startup-notification)
            ("unzip" ,unzip)
            ("zip" ,zip)
            ("zlib" ,zlib)))
         (native-inputs
          `(("perl" ,perl)
            ("python-2.7" ,python-2.7) ; Python 3 not supported
            ("python2-pysqlite" ,python2-pysqlite)
            ("yasm" ,yasm)
            ("pkg-config" ,pkg-config)
            ("autoconf" ,autoconf-2.13)
            ("which" ,which)
            ;;("rustc" ,rustc)
            ;;("cargo" ,cargo)))
            ))
         (home-page "https://mozilla.org/thunderbird/")
         (synopsis "Trademarkless version of Thunderbird")
         (description
          "Full-featured Email client built from Thunderbird source tree.")
         (license license:mpl2.0))) ;and others, see toolkit/content/license.html
